@@ -3,13 +3,14 @@ use strict;
 use Carp qw(croak);
 use Time::HiRes;
 use MogileFS::Exception;
+use MogileFS::DeviceState;
 
 require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT_OK = qw(
                     error undeferr debug fatal daemonize weighted_list every
                     wait_for_readability wait_for_writeability throw error_code
-                    max min
+                    max min first okay_args device_state
                     );
 
 sub every {
@@ -159,7 +160,7 @@ sub daemonize {
 #   my @items = weighted_list( [ 1, 10 ], [ 2, 20 ], [ 3, 0 ] );
 #
 #   returns (1, 2) or (2, 1) with the latter far more likely
-sub weighted_list {
+sub weighted_list (@) {
     my @list = grep { $_->[1] > 0 } @_;
     my @ret;
 
@@ -235,4 +236,25 @@ sub min {
     return $n2;
 }
 
+sub first (&@) {
+    my $code = shift;
+    foreach (@_) {
+        return $_ if $code->();
+    }
+    undef;
+}
+
+sub okay_args {
+    my ($href, @okay) = @_;
+    my %left = %$href;
+    delete $left{$_} foreach @okay;
+    return 1 unless %left;
+    Carp::croak("Unknown argument(s): " . join(", ", sort keys %left));
+}
+
+sub device_state {
+    my ($state) = @_;
+    return MogileFS::DeviceState->of_string($state);
+}
+
 1;
